@@ -68,6 +68,9 @@ class Bootstrapper:
         # package.
         self._seen_requirements: set[SeenKey] = set()
 
+        # Track requirements we have already resolved so we don't resolve them again.
+        self._resolved_requirements: dict[str, tuple[str, Version]] = {}
+
         self._build_order_filename = self.ctx.work_dir / "build-order.json"
 
     def resolve_version(
@@ -79,6 +82,11 @@ class Bootstrapper:
 
         Returns the source URL and the version of the requirement.
         """
+        req_str = str(req)
+        if req_str in self._resolved_requirements:
+            logger.debug(f"resolved {req_str} from cache")
+            return self._resolved_requirements[req_str]
+
         pbi = self.ctx.package_build_info(req)
         if pbi.pre_built:
             source_url, resolved_version = self._resolve_prebuilt_with_history(
@@ -90,9 +98,38 @@ class Bootstrapper:
                 req=req,
                 req_type=req_type,
             )
+
+        self._resolved_requirements[req_str] = (source_url, resolved_version)
         return source_url, resolved_version
 
+    def _processing_build_requirement(self, current_req_type: RequirementType) -> bool:
+        """Are we currently processing a build requirement?
+
+        We determine that a package is a build dependency if its requirement
+        type is build_system, build_backend, or build_sdist OR if it is an
+        installation requirement of something that is a build dependency. We
+        use a verbose loop to determine the status so we can log the reason
+        something is treated as a build dependency.
+        """
+        if current_req_type.is_build_requirement:
+            logger.debug(f"is itself a build requirement: {current_req_type}")
+            return True
+        if not current_req_type.is_install_requirement:
+            logger.debug(
+                "is not an install requirement, not checking dependency chain for a build requirement"
+            )
+            return False
+        for req_type, req, resolved_version in reversed(self.why):
+            if req_type.is_build_requirement:
+                logger.debug(
+                    f"is a build requirement because {req_type} dependency {req} ({resolved_version}) depends on it"
+                )
+                return True
+        logger.debug("is not a build requirement")
+        return False
+
     def bootstrap(self, req: Requirement, req_type: RequirementType) -> Version:
+        logger.info(f"bootstrapping {req} as {req_type} dependency of {self.why[-1:]}")
         constraint = self.ctx.constraints.get_constraint(req.name)
         if constraint:
             logger.info(
@@ -108,10 +145,17 @@ class Bootstrapper:
         self._add_to_graph(req, req_type, resolved_version, source_url)
 
         # Is bootstrap going to create a wheel or just an sdist?
-        # Use fast sdist-only if flag is set, requirement is not a build
-        # requirement, and wheel is not pre-built.
-        build_sdist_only = (
-            self.sdist_only and req_type.is_install_requirement and not pbi.pre_built
+        #
+        # Use fast sdist-only if flag is set and requirement is not a build
+        # requirement.
+        #
+        # An install requirement on a pre-built wheel treats the wheel as
+        # sdist-only in order to build its installation requirements sdist-only.
+        #
+        # When bootstrap encounters another package with a *build* requirement
+        # on a pre-built wheel, its installation dependencies are materialized.
+        build_sdist_only = self.sdist_only and not self._processing_build_requirement(
+            req_type
         )
 
         # Avoid cyclic dependencies and redundant processing.
@@ -259,12 +303,24 @@ class Bootstrapper:
             wheel_filename=wheel_filename,
         )
 
-        if build_sdist_only:
-            if typing.TYPE_CHECKING:
-                assert build_env is not None
-                assert sdist_root_dir is not None
-                assert wheel_filename is None
-
+        if wheel_filename is not None:
+            assert unpack_dir is not None
+            logger.debug(
+                "get install dependencies of wheel %s",
+                wheel_filename.name,
+            )
+            install_dependencies = dependencies.get_install_dependencies_of_wheel(
+                req=req,
+                wheel_filename=wheel_filename,
+                requirements_file_dir=unpack_dir,
+            )
+        elif sdist_filename is not None:
+            assert sdist_root_dir is not None
+            assert build_env is not None
+            logger.debug(
+                "get install dependencies of sdist from directory %s",
+                sdist_root_dir,
+            )
             install_dependencies = dependencies.get_install_dependencies_of_sdist(
                 ctx=self.ctx,
                 req=req,
@@ -272,15 +328,13 @@ class Bootstrapper:
                 build_env=build_env,
             )
         else:
-            if typing.TYPE_CHECKING:
-                assert wheel_filename is not None
-                assert unpack_dir is not None
+            # unreachable
+            raise RuntimeError("wheel_filename and sdist_filename are None")
 
-            install_dependencies = dependencies.get_install_dependencies_of_wheel(
-                req=req,
-                wheel_filename=wheel_filename,
-                requirements_file_dir=unpack_dir,
-            )
+        logger.debug(
+            "install dependencies: %s",
+            ", ".join(sorted(str(req) for req in install_dependencies)),
+        )
 
         self._add_to_build_order(
             req=req,
@@ -290,26 +344,19 @@ class Bootstrapper:
             prebuilt=pbi.pre_built,
             constraint=constraint,
         )
-        if req_type.is_build_requirement:
-            # install dependencies of build requirements are also build
-            # system requirements.
-            child_req_type = RequirementType.BUILD_SYSTEM
-        else:
-            # top-level and install requirements
-            child_req_type = RequirementType.INSTALL
 
         self.progressbar.update_total(len(install_dependencies))
         for dep in self._sort_requirements(install_dependencies):
             with req_ctxvar_context(dep):
                 try:
-                    self.bootstrap(req=dep, req_type=child_req_type)
+                    self.bootstrap(req=dep, req_type=RequirementType.INSTALL)
                 except Exception as err:
                     raise ValueError(f"could not handle {self._explain}") from err
             self.progressbar.update()
 
         # we are done processing this req, so lets remove it from the why chain
         self.why.pop()
-        self._cleanup(req, sdist_root_dir, build_env)
+        self.ctx.clean_build_dirs(sdist_root_dir, build_env)
         return resolved_version
 
     @property
@@ -463,17 +510,6 @@ class Bootstrapper:
         logger.info(f"{req_type} requirement {req} uses a pre-built wheel")
 
         wheel_filename = wheels.download_wheel(req, wheel_url, self.ctx.wheels_prebuilt)
-
-        # Add the wheel to the mirror so it is available to anything
-        # that needs to install it. We leave a copy in the prebuilt
-        # directory to make it easy to remove the wheel from the
-        # downloads directory before uploading to a proper package
-        # index.
-        dest_name = self.ctx.wheels_downloads / wheel_filename.name
-        if not dest_name.exists():
-            logger.info("updating temporary mirror with pre-built wheel")
-            shutil.copy(wheel_filename, dest_name)
-            server.update_wheel_mirror(self.ctx)
         unpack_dir = self._create_unpack_dir(req, resolved_version)
         return (wheel_filename, unpack_dir)
 
@@ -799,7 +835,9 @@ class Bootstrapper:
             wheel_url, resolved_version = cached_resolution
             logger.debug(f"resolved from previous bootstrap to {resolved_version}")
         else:
-            servers = wheels.get_wheel_server_urls(self.ctx, req)
+            servers = wheels.get_wheel_server_urls(
+                self.ctx, req, cache_wheel_server_url=resolver.PYPI_SERVER_URL
+            )
             wheel_url, resolved_version = wheels.resolve_prebuilt_wheel(
                 ctx=self.ctx, req=req, wheel_server_urls=servers, req_type=req_type
             )
@@ -888,26 +926,6 @@ class Bootstrapper:
         unpack_dir.mkdir(parents=True, exist_ok=True)
         return unpack_dir
 
-    def _cleanup(
-        self,
-        req: Requirement,
-        sdist_root_dir: pathlib.Path | None,
-        build_env: build_environment.BuildEnvironment | None,
-    ) -> None:
-        if not self.ctx.cleanup:
-            return
-
-        # Cleanup the source tree and build environment, leaving any other
-        # artifacts that were created.
-        if sdist_root_dir and sdist_root_dir.exists():
-            logger.debug(f"cleaning up source tree {sdist_root_dir}")
-            shutil.rmtree(sdist_root_dir)
-            logger.debug(f"cleaned up source tree {sdist_root_dir}")
-        if build_env and build_env.path.exists():
-            logger.debug(f"cleaning up build environment {build_env.path}")
-            shutil.rmtree(build_env.path)
-            logger.debug(f"cleaned up build environment {build_env.path}")
-
     def _add_to_graph(
         self,
         req: Requirement,
