@@ -4,10 +4,12 @@ from . import (
     build_order,
     canonicalize,
     download_sequence,
+    find_updates,
     graph,
     lint,
     lint_requirements,
     list_overrides,
+    list_versions,
     migrate_config,
     minimize,
     server,
@@ -17,13 +19,16 @@ from . import (
 
 commands = [
     bootstrap.bootstrap,
+    bootstrap.bootstrap_parallel,
     build.build,
     build.build_sequence,
     build.build_parallel,
     build_order.build_order,
+    find_updates.find_updates,
     graph.graph,
     lint.lint,
     list_overrides.list_overrides,
+    list_versions.list_versions,
     migrate_config.migrate_config,
     minimize.minimize,
     stats.stats,
