@@ -34,6 +34,7 @@ from fromager import (
     wheels,
 )
 
+from .. import resolver
 from ..log import VERBOSE_LOG_FMT, ThreadLogFilter, req_ctxvar_context
 
 logger = logging.getLogger(__name__)
@@ -122,7 +123,7 @@ def build(
             req=req,
             source_download_url=source_url,
             force=True,
-            wheel_server_urls=[wkctx.wheel_server_url],
+            cache_wheel_server_url=None,
         )
     print(wheel_filename)
 
@@ -163,16 +164,16 @@ def build_sequence(
 
     """
     server.start_wheel_server(wkctx)
-    wheel_server_urls = [wkctx.wheel_server_url]
-    if cache_wheel_server_url:
-        # put after local server so we always check local server first
-        wheel_server_urls.append(cache_wheel_server_url)
 
     if force:
-        logger.info(f"rebuilding all wheels even if they exist in {wheel_server_urls}")
+        logger.info(
+            "rebuilding all wheels even if they exist in "
+            f"{wkctx.wheel_server_url=}, {cache_wheel_server_url=}"
+        )
     else:
         logger.info(
-            f"skipping builds for versions of packages available at {wheel_server_urls}"
+            "skipping builds for versions of packages available at "
+            f"{wkctx.wheel_server_url=}, {cache_wheel_server_url=}"
         )
 
     entries: list[BuildSequenceEntry] = []
@@ -201,7 +202,7 @@ def build_sequence(
                     req=req,
                     source_download_url=source_download_url,
                     force=force,
-                    wheel_server_urls=wheel_server_urls,
+                    cache_wheel_server_url=cache_wheel_server_url,
                 )
                 if prebuilt:
                     logger.info("downloaded prebuilt wheel %s", wheel_filename)
@@ -326,7 +327,7 @@ def _build(
     req: Requirement,
     source_download_url: str,
     force: bool,
-    wheel_server_urls: list[str] | None = None,
+    cache_wheel_server_url: str | None,
 ) -> tuple[pathlib.Path, bool]:
     """Handle one version of one wheel.
 
@@ -335,9 +336,7 @@ def _build(
     2. Download a pre-built wheel.
     3. Build the wheel from source.
     """
-
-    wheel_server_urls = wheel_server_urls or []
-    wheel_filename = None
+    wheel_filename: pathlib.Path | None = None
 
     # Set up a log file for all of the details of the build for this one wheel.
     # We attach a handler to the root logger so that all messages are logged to
@@ -352,20 +351,26 @@ def _build(
     root_logger.addHandler(file_handler)
 
     logger.info("starting processing")
+    pbi = wkctx.package_build_info(req)
+    prebuilt = pbi.pre_built
 
+    wheel_server_urls = wheels.get_wheel_server_urls(
+        wkctx, req, cache_wheel_server_url=cache_wheel_server_url
+    )
+
+    # See if we can reuse an existing wheel.
     if not force:
-        is_built, wheel_filename = _is_wheel_built(
+        wheel_filename = _is_wheel_built(
             wkctx,
             req.name,
             resolved_version,
             wheel_server_urls,
         )
-        if is_built:
-            logger.info("found existing wheel %s", wheel_filename)
+        if wheel_filename:
+            logger.info("using existing wheel from %s", wheel_filename)
 
-    pbi = wkctx.package_build_info(req)
-    prebuilt = pbi.pre_built
-    if prebuilt:
+    # See if we can download a prebuilt wheel.
+    if prebuilt and not wheel_filename:
         logger.info("downloading prebuilt wheel")
         wheel_filename = wheels.download_wheel(
             req=req,
@@ -379,10 +384,6 @@ def _build(
             dist_version=str(resolved_version),
             wheel_filename=wheel_filename,
         )
-        server.update_wheel_mirror(wkctx)
-        # After we update the wheel mirror, the built file has
-        # moved to a new directory.
-        wheel_filename = wkctx.wheels_downloads / wheel_filename.name
 
     # If we get here and still don't have a wheel filename, then we need to
     # build the wheel.
@@ -440,6 +441,8 @@ def _build(
             wheel_filename=wheel_filename,
         )
 
+        wkctx.clean_build_dirs(source_root_dir, build_env)
+
     root_logger.removeHandler(file_handler)
     file_handler.close()
 
@@ -457,23 +460,26 @@ def _is_wheel_built(
     dist_name: str,
     resolved_version: Version,
     wheel_server_urls: list[str],
-) -> tuple[bool, pathlib.Path | None]:
+) -> pathlib.Path | None:
     req = Requirement(f"{dist_name}=={resolved_version}")
 
     try:
         logger.info(
-            "checking if a suitable wheel was already built on %s", wheel_server_urls
+            "checking if a suitable wheel for %s was already built on %s",
+            req,
+            wheel_server_urls,
         )
         url, _ = wheels.resolve_prebuilt_wheel(
             ctx=wkctx,
             req=req,
             wheel_server_urls=wheel_server_urls,
         )
+        logger.info("found candidate wheel %s", url)
         pbi = wkctx.package_build_info(req)
         build_tag_from_settings = pbi.build_tag(resolved_version)
         build_tag = build_tag_from_settings if build_tag_from_settings else (0, "")
-        wheel_filename = urlparse(url).path.rsplit("/", 1)[-1]
-        _, _, build_tag_from_name, _ = parse_wheel_filename(wheel_filename)
+        wheel_basename = resolver.extract_filename_from_url(url)
+        _, _, build_tag_from_name, _ = parse_wheel_filename(wheel_basename)
         existing_build_tag = build_tag_from_name if build_tag_from_name else (0, "")
         if (
             existing_build_tag[0] > build_tag[0]
@@ -482,16 +488,36 @@ def _is_wheel_built(
             raise ValueError(
                 f"{dist_name}: changelog for version {resolved_version} is inconsistent. Found build tag {existing_build_tag} but expected {build_tag}"
             )
-        is_built = existing_build_tag == build_tag
-        if is_built and wkctx.wheel_server_url not in url:
-            # if the found wheel was on an external server, then download it
-            wheels.download_wheel(req, url, wkctx.wheels_downloads)
-            server.update_wheel_mirror(wkctx)
+        if existing_build_tag != build_tag:
+            logger.info(
+                f"candidate wheel build tag {existing_build_tag} does not match expected build tag {build_tag}"
+            )
+            return None
 
-        return is_built, pathlib.Path(wheel_filename)
+        wheel_filename: pathlib.Path | None = None
+        if url.startswith(wkctx.wheel_server_url):
+            logging.debug("found wheel on local server")
+            wheel_filename = wkctx.wheels_downloads / wheel_basename
+            if not wheel_filename.exists():
+                logger.info("wheel not found in local cache, preparing to download")
+                wheel_filename = None
+
+        if not wheel_filename:
+            # if the found wheel was on an external server, then download it
+            logger.info("downloading wheel from %s", url)
+            wheel_filename = wheels.download_wheel(req, url, wkctx.wheels_downloads)
+
+        return wheel_filename
     except Exception:
+        logger.debug(
+            "could not locate prebuilt wheel %s-%s on %s",
+            dist_name,
+            resolved_version,
+            wheel_server_urls,
+            exc_info=True,
+        )
         logger.info("could not locate prebuilt wheel")
-        return False, None
+        return None
 
 
 def _build_parallel(
@@ -500,7 +526,7 @@ def _build_parallel(
     req: Requirement,
     source_download_url: str,
     force: bool,
-    wheel_server_urls: list[str],
+    cache_wheel_server_url: str | None,
 ) -> tuple[pathlib.Path, bool]:
     """
     This function runs in a thread to manage the build of a single package.
@@ -512,7 +538,7 @@ def _build_parallel(
             req=req,
             source_download_url=source_download_url,
             force=force,
-            wheel_server_urls=wheel_server_urls,
+            cache_wheel_server_url=cache_wheel_server_url,
         )
 
 
@@ -669,7 +695,7 @@ def build_parallel(
                         req=req,
                         source_download_url=node.download_url,
                         force=force,
-                        wheel_server_urls=wheel_server_urls,
+                        cache_wheel_server_url=cache_wheel_server_url,
                     )
                     future.add_done_callback(update_progressbar_cb)
                     futures.append(future)
