@@ -731,6 +731,7 @@ class PackageBuildInfo:
         return sdist_root_dir
 
     def get_changelog(self, version: Version) -> list[str]:
+        """Get changelog for a version"""
         # ignore local version for changelog entries
         version = Version(version.public)
         pv = typing.cast(PackageVersion, version)
@@ -747,6 +748,9 @@ class PackageBuildInfo:
            the build tag from changelog, e.g. version `1.0.3+local.suffix`
            uses `1.0.3`.
         """
+        if self.pre_built:
+            # pre-built wheels have no built tag
+            return ()
         pv = typing.cast(PackageVersion, version)
         release = len(self.get_changelog(pv))
         if release == 0:
@@ -858,6 +862,11 @@ class PackageBuildInfo:
     def exclusive_build(self) -> bool:
         return self._ps.build_options.exclusive_build
 
+    @property
+    def variants(self) -> Mapping[Variant, VariantInfo]:
+        """Get the variant configuration for the current package"""
+        return self._ps.variants
+
     def serialize(self, **kwargs) -> dict[str, typing.Any]:
         return self._ps.serialize(**kwargs)
 
@@ -1063,3 +1072,13 @@ class Settings:
             packages.add(Package(canonicalize_name(name, validate=True)))
 
         return packages
+
+    def all_variants(self) -> set[Variant]:
+        """List all variants with overrides"""
+        variants: set[Variant] = set()
+        # from global settings
+        variants.update(self._settings.changelog.keys())
+        # from package settings
+        for ps in self._package_settings.values():
+            variants.update(ps.variants.keys())
+        return variants
